MAX_PAGES_ALLOWED_BY_API = 20

APPLICATIONS = 'applications'
ASSETS = 'assets'
ASSET_GROUPS = 'asset_groups'
CONNECTORS = 'connectors'
CONNECTOR_RUNS = 'connector_runs'
DASHBOARD_GROUPS = 'dashboard_groups'
FIXES = 'fixes'
ROLES = 'roles'
USERS = 'users'
VULNERABILITIES = 'vulnerabilities'
VULNERABILITY_DEFINITIONS = 'vulnerability_definitions'

COLLECTION_NAMES = [
    APPLICATIONS,
    ASSETS,
    ASSET_GROUPS,
    CONNECTORS,
    CONNECTOR_RUNS,
    DASHBOARD_GROUPS,
    FIXES,
    ROLES,
    USERS,
    VULNERABILITIES,
    VULNERABILITY_DEFINITIONS,
]

APPLICATION = 'application'
ASSET = 'asset'
ASSET_GROUP = 'asset_group'
CONNECTOR = 'connector'
CONNECTOR_RUN = 'connector_run'
DASHBOARD_GROUP = 'dashboard_group'
FIX = 'fix'
ROLE = 'role'
USER = 'user'
VULNERABILITY = 'vulnerability'
VULNERABILITY_DEFINITION = 'vulnerability_definition'

OBJECT_TYPES = [
    APPLICATION,
    ASSET,
    ASSET_GROUP,
    CONNECTOR,
    CONNECTOR_RUN,
    DASHBOARD_GROUP,
    FIX,
    ROLE,
    USER,
    VULNERABILITY,
    VULNERABILITY_DEFINITION,
]

